package jalview.gui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import jalview.datamodel.*;
import jalview.schemes.*;
import jalview.analysis.*;


public class SeqCanvas extends JPanel
{
    Image             img;
    Graphics          gg;
    int               imgWidth;
    int               imgHeight;

    AlignViewport     av;

    public boolean paintFlag = false;

    boolean showScores = false;
    boolean displaySearch = false;
    int [] searchResults = null;

    int chunkHeight;
    int chunkWidth;


    public SeqCanvas(AlignViewport av)
    {
	this.av         = av;
       setLayout(new BorderLayout());
       PaintRefresher.Register(this);

    }

  public void drawScale(int startx, int endx,int charWidth, int charHeight,int ypos) {
      int scalestartx = startx - startx%10 + 10;

      gg.setColor(Color.black);

      for (int i=scalestartx;i < endx;i+= 10) {
	  String string = String.valueOf(i);
	  gg.drawString(string,(int)((i-startx-1)*charWidth),ypos+15 - charHeight*(2));
      }
  }


/**
 * Definitions of startx and endx (hopefully):
 * SMJS This is what I'm working towards!
 *   startx is the first residue (starting at 0) to display.
 *   endx   is the last residue to display (starting at 0).
 *   starty is the first sequence to display (starting at 0).
 *   endy   is the last sequence to display (starting at 0).
 * NOTE 1: The av limits are set in setFont in this class and
 * in the adjustment listener in SeqPanel when the scrollbars move.
 */

  public void paintComponent(Graphics g) {

    AlignmentI da = av.getAlignment();

    if (img == null ||
        imgWidth  !=  getWidth()  ||
        imgHeight !=  getHeight()
        || paintFlag)
    {

      imgWidth  = getWidth();
      imgHeight = getHeight();

      img = createImage(imgWidth,imgHeight);
      gg  = img.getGraphics();

      gg.setFont(av.getFont());

      paintFlag = false;
    }

    chunkWidth  =   getWidth()/av.charWidth;
    chunkHeight =  (da.getHeight() + 2)*av.charHeight;

    av.setChunkHeight(chunkHeight);
    av.setChunkWidth(chunkWidth);



    fillBackground(gg,Color.WHITE,0,0,imgWidth,imgHeight);

    /* if (av.getWrapAlignment()) {
        int offy = av.getStartSeq();
	  startx = (int)(offy/chunkWidth)*chunkWidth;
	  endx   = startx + chunkWidth;
	  starty = offy%chunkHeight;
	  endy   = starty + da.getHeight();

	  int ypos     = 0;
	  int rowstart = starty;

	  if (starty == 0) {
	      ypos = 2*charHeight;
	  } else if (starty == 1) {
	      starty = 0;
	      ypos = charHeight;
	  }

	  if (endy > da.getHeight()) {
	      endy = da.getHeight();
	  }

	  if (endx > da.getWidth()) {
	      endx = da.getWidth();
	  }

	  if (rowstart < 2) {
	      drawScale(startx,endx,charWidth,charHeight,ypos);
	  }

	  drawPanel(gg,startx,endx,starty,endy,startx,starty,ypos);

	  if (rowstart == 0) {
	      ypos = ypos + chunkHeight;
	  } else if (rowstart == 1) {
	      ypos = ypos + chunkHeight;
	  } else {
	      ypos   = ypos + chunkHeight - rowstart*charHeight;
	  }

	  startx += chunkWidth;
	  endx   = startx + chunkWidth;
	  starty = 0;

	  if (endx > da.getWidth()) {
	      endx = da.getWidth();
	  }
	  // Draw the rest of the panels

	  while (ypos <= getHeight()) {
	      drawScale(startx,endx,charWidth,charHeight,ypos);
	      drawPanel(gg,startx,endx,0,da.getHeight(),startx,starty,ypos);

	      ypos   += chunkHeight;
	      startx += chunkWidth;
	      endx   = startx + chunkWidth;

	      if (endy > da.getHeight()) {
		  endy = da.getHeight();
	      }

	      if (endx > da.getWidth()) {
		  endx = da.getWidth();
	      }

	  }
      }
      else*/
      {
	  drawPanel(gg,av.startRes,av.endRes,av.startSeq,av.endSeq,av.startRes,av.startSeq,0);
      }


  /*  if ((oldendy -oldstarty) > (getWidth() / av.getCharWidth())) {
      System.out.println("LIMITS ERROR LIMITS ERROR");
      System.out.println("Corrds " + (oldendy-oldstarty) + " " + (getWidth()/av.getCharWidth()) + " " + getWidth() + " " + av.getCharWidth());
    }*/

    g.drawImage(img,0,0,this);

  }




  public void drawPanel(Graphics g,int x1,int x2, int y1, int y2,int startx, int starty,int offset) {


    g.setFont(av.getFont());
    int            charWidth  = av.getCharWidth();
    int            charHeight = av.getCharHeight();
    RendererI sr = av.getRenderer();

    /*Vector    pid    = av.getConsensus(false);
    Vector tmpseq = new Vector();
    for (int i = 0; i < av.getAlignment().getHeight(); i++)
	if (!av.getSelection().contains(av.getAlignment().getSequenceAt(i)))
	    tmpseq.addElement(av.getAlignment().getSequenceAt(i));

    if (sr instanceof SequenceRenderer)
	pid    = AAFrequency.calculate(tmpseq,x1,x2);

    else if (sr instanceof GraphRenderer)
	pid = AAFrequency.calculatePID(av.getAlignment().getSequenceAt(0),
				       av.getAlignment().getSequences(),
				       av.getPIDWindow(),x1,x2);

*/

    if (y2 > starty && y1 < av.getEndSeq())
    {
       fillBackground(g,
                   Color.red,
                   (x1-startx)*charWidth,
                   offset + AlignmentUtil.getPixelHeight(starty,y1,av.getCharHeight()),
                   (x2-x1+1)*charWidth,
                   offset + AlignmentUtil.getPixelHeight(y1,y2,av.getCharHeight()));
    }

    SequenceI nextSeq;
    SequenceGroup group=null;
    boolean inGroup=false; // are we in a defined group?
    boolean inRB=false; //Are we in a rubberband group?
    int sx=-1, sy=-1, ex=-1, ey=-1;
    int oldStartX=0,oldEndX=0,oldY=0;
    int oldRStartX=0,oldREndX=0,oldRY=0;
    for (int i = y1 ; i < y2 ;i++)
    {
     nextSeq = av.getAlignment().getSequenceAt(i);

     group = av.alignment.findGroup( nextSeq );

     sr.drawSequence(g, nextSeq, group,x1,x2,
                 (x1 - startx) * charWidth,
                 offset + AlignmentUtil.getPixelHeight(starty, i, av.getCharHeight()),
                 charWidth,charHeight,null, i);


     if( group!=null )
     {

        g.setColor(Color.lightGray);

        sx = (group.getStartRes()-startx)*charWidth;
        sy = offset + AlignmentUtil.getPixelHeight(starty, i, av.getCharHeight());
        ex = (group.getEndRes()+1-startx)*charWidth;
        ey = offset + AlignmentUtil.getPixelHeight(starty, i+1, av.getCharHeight());


        if (!inGroup)
        {
          g.drawLine(sx, sy, ex, sy); // Horizontal, top of new box
          inGroup=true;
        }

        g.drawLine(sx, sy, sx, ey ); // vertical line, left hand side
        g.drawLine( ex,sy,ex,ey);    // vertical line, right hand side
        if (i == y2 - 1)// last line of alignment, seal the box
          g.drawLine(sx, ey, ex, ey); // Horizontal, bottom of old box

        oldStartX = sx;
        oldEndX = ex;
        oldY = ey;

     }
     else if(inGroup)
     {
       g.setColor(Color.lightGray);
       g.drawLine(oldStartX, oldY, oldEndX, oldY); // Horizontal, bottom of old box
     }
     else
       inGroup = false;

     group = av.getRubberbandGroup();
     if( group != null && group.sequences.contains(nextSeq) )
     {
         g.setColor(Color.RED.brighter());
         sx = (group.getStartRes() - startx) * charWidth;
         sy = offset +
             AlignmentUtil.getPixelHeight(starty, i, av.getCharHeight());
         ex = (group.getEndRes() + 1 - startx) * charWidth;
         ey = offset +
             AlignmentUtil.getPixelHeight(starty, i + 1, av.getCharHeight());

         if (!inRB)
         {

           g.drawLine(sx, sy, ex, sy); // Horizontal, top of new box
           inRB = true;
         }

         g.drawLine(sx, sy, sx, ey); // vertical line, left hand side
         g.drawLine(ex, sy, ex, ey); // vertical line, right hand side
         if (i == y2 - 1) // last line of alignment, seal the box
           g.drawLine(sx, ey, ex, ey); // Horizontal, bottom of old box

          oldRStartX = sx;
          oldREndX = ex;
          oldRY = ey;
     }
     else if(inRB)
     {
       g.setColor(Color.RED.brighter());
       g.drawLine(oldRStartX, oldRY, oldREndX, oldRY); // Horizontal, bottom of old box
       inRB = false;
     }
     else
       inRB = false;

    }

    /// Highlight search Results once all sequences have been drawn
    if(displaySearch)
    {
      for(int r=0; r<searchResults.length; r+=3)
      {
        int searchSeq = searchResults[r];
        int searchStart = searchResults[r+1];
        int searchEnd = searchResults[r+2];

        if (searchSeq >= y1 && searchSeq <= y2)
        {
          SequenceRenderer ssr = (SequenceRenderer) sr;
          ssr.drawHighlightedText(g, av.getAlignment().getSequenceAt(searchSeq),
                                  searchStart,
                                  searchEnd,
                                  (searchStart - startx) * charWidth,
                                  offset +
                                  AlignmentUtil.getPixelHeight(starty, searchSeq,
              charHeight),
                                  charWidth,
                                  charHeight);
        }
      }
    }

  }

  public void fillBackground(Graphics g,Color c, int x1,int y1,int width,int height) {
    g.setColor(c);
    g.fillRect(x1,y1,width,height);
  }

  public int getChunkWidth() {
    return chunkWidth;
  }

  public void highlightSearchResults(int [] results)
  {
    // results are in the order sequence, startRes, endRes
    if(results==null)
      displaySearch = false;
    else
      displaySearch = true;

    searchResults = results;

    paintFlag = true;
    repaint();
  }


}
